//-----------------------------------------------------------------------------
//
// ClassName:    IHistogram
//  
// Description:  Definition of the IHistogram interface class
//
//               IHistogram is a base class for all other histogram interfaces.
//               It contains their common functionality.
//
//               IHistogram is not a complete histogram interface, it cannot
//               be used as a standard histogram.  It is ment to be used e.g.
//               for histogram managment (there, where the number of axes is
//               not important).
//
// Design principles :
//
//  1) Only interfaces, basic types, and types from foundation libraries
//     are allowed (STL is currently the only one foundation library)
//     - Uses only int and double as basic types
//     - Internals (classes, enums, or others) of concrete packages
//       do not appear in the interface
//  2) Accessors - XXX() (no getXXX() functions)
//  3) In member function names
//     - in-range bins are not specified explicitly (e.g. sumBinHeights())
//     - ...All... mean all (in-range and extra) bins (e.g. sumAllBinHeights())
//     - ...Extra... mean extra bins (e.g. sumExtraBinHeights())
//
//  The AIDA group.
//
// 04/02/2000 : Pavel Binko, Dino Ferrero Merlino, Andreas Pfeiffer :
//   First version.
// 03/11/2000 : G.Barrand : 
//   typedef std_string, AIDA_DONT_USE_STD.
//   Move [min, max]BinHeight() from 1D, 2D here.
//-----------------------------------------------------------------------------


#ifndef IHistogram_h
#define IHistogram_h

#include <string>
#ifdef AIDA_DONT_USE_STD
typedef string std_string;
#else
typedef std::string std_string;
#endif

class IAxis;
class IAnnotation;


class IHistogram {

public:
  virtual ~IHistogram() { /* nop */ }

public:


  /// =========================================================================
  /// Underflow and overflow bins
  /// =========================================================================

  /// -------------------------------------------------------------------------
  /// Constant specifying the underflow and overflow bin
  ///   (can be passed to any method expecting a bin index)
  enum { UNDERFLOW_BIN = -2, OVERFLOW_BIN = -1 };


  /// =========================================================================
  /// Histogram characteristics
  /// =========================================================================

  /// -------------------------------------------------------------------------
  /// Title of the histogram (will be set only in the constructor)
  virtual std_string title() const = 0;

  /// -------------------------------------------------------------------------
  /// Annotation of the histogram
  virtual IAnnotation* annotation() = 0;

  /// -------------------------------------------------------------------------
  /// Number of dimensions (1 for 1D histogram, 2 for 2D histogram, etc.)
  virtual int dimensions() const = 0;

  /// -------------------------------------------------------------------------
  /// Reset contents
  virtual void reset() = 0;


  /// =========================================================================
  /// Total number of entries in the whole histogram (number of fills)
  /// =========================================================================

  /// Number of in-range entries in the whole histogram
  virtual int entries() const = 0;

  /// Number of all entries in all (both in-range and extra) bins
  ///   in the whole histogram
  virtual int allEntries() const = 0;

  /// Number of extra entries in the whole histogram
  virtual int extraEntries() const = 0;

  /// Number of equivalent entries
  ///   SUM[ weight ] ^ 2 / SUM[ weight^2 ]
  virtual double equivalentBinEntries() const = 0;


  /// =========================================================================
  /// Sums of bin heights (height respects weights of entries)
  /// =========================================================================

  /// Sum of in-range bin heights in the whole histogram
  virtual double sumBinHeights() const = 0;

  /// Sum of all (both in-range and extra) bin heights in the whole histogram
  virtual double sumAllBinHeights() const = 0;

  /// Sum of extra bin heights in the whole histogram
  virtual double sumExtraBinHeights() const = 0;

  /// Min height of in-range bins in the whole histogram
  virtual double minBinHeight() const = 0;

  /// Max height of in-range bins in the whole histogram
  virtual double maxBinHeight() const = 0;
};


#endif
