//-----------------------------------------------------------------------------
//
// ClassName:    IHistogram1D
//  
// Description:  Definition of the IHistogram1D interface class
//
//           All methods that accept a bin number as an argument will
//           also accept the UNDERFLOW or OVERFLOW as the argument,
//           and as a result will give the contents of the resulting
//           UNDERFLOW or OVERFLOW bin.
//           The in-range bin indeces are expected to be from 0 to N-1.
//           See also <a href="http://wwwinfo.cern.ch/asd/lhc++/AIDA/">AIDA</a>
//
//  The AIDA group.
//
// 04/02/2000 : Pavel Binko, Dino Ferrero Merlino, Andreas Pfeiffer :
//   First version.
// 03/11/2000 : G.Barrand : 
//   Return const reference for xAxis().
//   SunOS-CC-5, HP-UX-aCC : inheriting "virtual" induces problems.
//   Move [min, max]BinHeight() in IHistogram.
// 07/11/2000 : G.Barrand : 
//   CPP macro AIDA_DONT_USE_VIRTUAL_INHERITANCE to handle that problems :
//    default is to have virtual inheritance.
//-----------------------------------------------------------------------------


#ifndef IHistogram1D_h
#define IHistogram1D_h

#include <IHistogram.h>

class IHistogram1D : 
#ifndef AIDA_DONT_USE_VIRTUAL_INHERITANCE
virtual
#endif
public IHistogram {

public:
  virtual ~IHistogram1D() { /* nop */ }

public:


  /// =========================================================================
  /// Histogram characteristics
  /// =========================================================================

  /// -------------------------------------------------------------------------
  /// Fill histogram
  virtual void fill( double x, double weight = 1 ) = 0;


  /// =========================================================================
  /// Entries
  ///   The index accepts UNDERFLOW, values from 0 to N-1, or OVERFLOW
  /// =========================================================================

  /// Number of entries in a specific in-range bin (number of fills)
  virtual int binEntries( int index ) const = 0;

  /// Height of a specific in-range bin (height respects weights of entries)
  virtual double binHeight( int index ) const = 0;

  /// Bin contents error
  virtual double binError( int index ) const = 0;


  /// =========================================================================
  /// Statistics
  /// =========================================================================

  /// mean() returns the mean of the whole histogram
  ///   as calculated on filling-time
  virtual double mean() const = 0;

  /// rms() returns the rms of the whole histogram
  ///   as calculated on filling-time
  virtual double rms() const = 0;


  /// =========================================================================
  /// Minimum and maximum heights and bins
  /// =========================================================================

  /// Index of the in-range bin containing the minBinHeight()
  virtual int minBin() const = 0;

  /// Index of the in-range bin containing the maxBinHeight()
  virtual int maxBin() const = 0;


  /// =========================================================================
  /// Axis
  /// =========================================================================

  /// Get the X axis
  virtual const IAxis& xAxis() const = 0;


  /// =========================================================================
  /// Conversions between coordinates and bin indices
  /// =========================================================================

  /// Equivalent to xAxis().coordToIndex( coord );
  virtual int coordToIndex( double coord ) const = 0;

};


#endif
