package hep.aida;

/**
 * User level interface to Plotter.
 * This IPlotter interface assumes the handling of 
 * multiple plotting regions. Then the
 * IPlotter should be seen as a "page" managing multiple
 * drawing region (and 
 * should probably inherit some IPage interface).
 * It assumes that a "current" region mechanism is used.
 * The keyword "region" is borrowed 
 * from the OpenInventor terminology (viewing region).
 * 
 * @author Guy Barrand
 * @version $Id: IPlotter.java,v 1.4 2001/11/29 10:22:48 barrand Exp $
 */
public interface IPlotter {
    
  /**
   * createRegion, createRegions are the region factories.
   */
  /**
   * Create a new region.
   * @param x, y, w, h : position and size in normal coordinates
   * that is to say between 0 and 1. Then a region with (x=0,y=0,w=1,h=1) maps
   * the full page. "x" goes left to right. "y" bottom to top.
   * @return The index of the region. IConstants::NOT_FOUND is returned in case
   * of failure.
   */
  public int createRegion(double x,double y, double w, double h);

  /**
   * Create a grid of regions.
   * By default the current region is the top left one (with index 0).
   * Region are indexed from zero, with increasing order
   * left to right, top to bottom.
   */
  public boolean createRegions(int columns, int rows, int start);

  /**
   * Add a data analysis object (histogram, function, cloud,...)
   * in the list of things to plot in the current region.
   * Activate the underlying graphic toolkit in order to bring
   * something in the screen window(s) attached to the plotter.
   * It assumes that the plotter had been mapped on the
   * screen with the show method.
   */
  public boolean plot(IHistogram histogram);
  public boolean plot(IFunction function);
  public boolean plot(ICloud cloud);

  /**
   * Remove a data analysis object in the list of things to plot 
   * in the current region.
   * Activate the graphic layer to update the screen window.
   */
  public boolean remove(IHistogram histogram);
  public boolean remove(IFunction function);
  public boolean remove(ICloud cloud);
    
  /**
   * Set current region to be the "next" one.
   * If exceeding the number of regions, 
   * the current region index return to 0.
   */ 
  public boolean next();

  /**
   * Set current region by giving its index [0,n-1]
   */ 
  public boolean setCurrentRegion(int index);

  /**
   * @return current region index.
   */ 
  public int currentRegion();

  /**
   * Clear the list of things to plot for the current region.
   * Clear the corresponding area on screen window(s).
   * The clearing of all regions could be done by reissuing a 
   * setRegions.
   */
  public boolean clearRegion();
    
  /**
   *  Clear all regions. It does not reset the number 
   * and position of regions. This can be done through
   * a setRegions(1,1,0).
   */
  public boolean clearPage();

  /*
   * Set various plotting paramters for the current region
   * or the whole page.
   * Activate the graphic layer and update the screen window(s) 
   * if needed.
   */
  public boolean setParameter(String parameter, String value);
    
  /**
   * Produce an output file.
   * @param type  A string to describe the type of the output : 
   *  "PS"  or "PostScript" for PostScript, "JPEG" for JPEG.
   *  If nothing is given, the type is guessed
   *  from the file suffix (.ps for PostScript,
   *  .jpg for JPEG, etc...).
   */
  public boolean write(String filename, String type);

  /*
   * GUI related methods :
   */

  /**
   * Map the plotter on the screen.
   */
  public boolean show();
    
  /**
   * Unmap the plotter on the screen.
   */
  public boolean hide();

  /**
   * Refresh the screen window(s).
   * Used when a plotted data analysis object
   * received new data.
   */
  public boolean refresh();
    
  /**
   * Give control to the plotter GUI. The plotter must be able 
   * to return from this function from a user action in its GUI.
   */
  public boolean interact();
    
}

