package hep.aida.dev;
import hep.aida.*;

/**
 * A class for N-dimensional Histograms.
 *
 * One of the 4 fill/fillWeighted methods is called, depending on the
 * dimension of the histogram.
 * The split of these 4 fill/fillWeighted methods was done for
 * performance reasons.
 *
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version 2.0, 06-02-2001
 */
public interface IStatisticsHistogram extends IConstants
{
   /**
    * Fill histogram with weight 1.
    * This method is called for 1D histograms.
    */
    public void fill(double x);
   /**
    * Fill histogram with specified weight.
    * This method is called for 1D histograms.
    */
    public void fillWeighted(double x, double weight);
   /**
    * Fill histogram with weight 1.
    * This method is called for 2D histograms.
    */
    public void fill(double x, double y);
   /**
    * Fill histogram with specified weight.
    * This method is called for 2D histograms.
    */
    public void fillWeighted(double x, double y, double weight);
   /**
    * Fill histogram with weight 1.
    * This method is called for 3D histograms.
    */
    public void fill(double x, double y, double z);
   /**
    * Fill histogram with specified weight.
    * This method is called for 3D histograms.
    */
    public void fillWeighted(double x, double y, double z, double weight);
   /**
    * Fill histogram with weight 1.
    * This method is called for > 3D histograms.
    */
    public void fill(double[] x);
   /**
    * Fill histogram with specified weight.
    * This method is called for > 3D histograms.
    */
    public void fillWeighted(double[] x, double weight);
   /**
    * Label of the histogram (will be set only in the constructor).
    */
    public String label();
   /**
    * to be defined
    */
    public IAnnotation annotation();
   /**
    * Returns the dimension of the histogram.
    */
    public int dimension();
   /**
    * Reset contents; as if just constructed.
    */
    public void reset();
    
   /**
    * Number of in-range entries in the histogram.
    */
    public int entries();
   /**
    * Number of all entries in all (both in-range and under/overflow) bins in the histogram.
    */
    public int allEntries();
   /**
    * Number of under and overflow entries in the histogram.
    */
    public int extraEntries();
   /**
    * Number of equivalent entries.
    * @return <tt>SUM[ weight ] ^ 2 / SUM[ weight^2 ]</tt>.
    */
    public double equivalentBinEntries();
   /**
    * Sum of in-range bin heights in the histogram.
    */
    public double sumBinHeights();
    /**
     * Sum of all (both in-range and under/overflow) bin heights in the histogram.
     */
    public double sumAllBinHeights();
   /**
    * Sum of under/overflow bin heights in the histogram.
    */
    public double sumExtraBinHeights();
   /**
    * Minimum height of in-range bins in the histogram.
    */
    public double minBinHeight();
   /**
    * Maximum height of in-range bins in the histogram.
    */
    public double maxBinHeight();
   /**
    * Number of entries in the corresponding bin (ie the number of times fill was called for this bin).
    * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
    */
    public int binEntries(int[] index);
   /**
    * Total height of the corresponding bin (ie the sum of the weights in this bin).
    * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
    */
    public double binHeight(int[] index);
   /**
    * The error on this bin.
    * @param index the bin number (0...N-1) or OVERFLOW or UNDERFLOW.
    */
    public double binError(int[] index);
   /**
    * Returns the mean of the whole histogram as calculated on filling-time.
    */
    public double mean(int axis);
   /**
    * Returns the rms of the whole histogram as calculated on filling-time.
    */
    public double rms(int axis);
   /**
    * Returns the axis.
    */
    public IAxis axis(int axis);
}
