#!/usr/bin/env python2.0

#
# ap, 07-jul-2001: version 1.0
#                  initial release for AIDA-2.0 classes.
#                  open issues with mappings for Class, Object and String[]
#
# ap, 07-aug-2001: version 1.1
#                  release for AIDA 2.1 classes
#                  mapping now "Class" to "const std::string &"

import sys, os, string, re

class Java2CPPConverter :

    "converts AIDA files written in Java into their corresponding C++ header files"
    
    def __init__ (self) :
        # reset the internal arrays ... needed if processing more than one file
        self.reset()

        # define the regular expressions for the various parts 
        self.classDecl      = re.compile(r'(\s*)public interface\s*I(?P<clName>[a-zA-Z]\w*\s*)(\s*(extends\s*)(I(?P<baseNames>[a-zA-Z]\w*)))?\s*{')
        self.classPartDecl  = re.compile(r'(\s*)public interface\s*I(?P<clName>[a-zA-Z]\w*\s*)(\s*(extends\s*)(I(?P<baseNames>[a-zA-Z]\w*)))?\s*')
        self.classEnd       = re.compile(r'[\n\s]*}[\s\n]*')
        self.removeDecl     = re.compile(r'\s*(import|package)\s*.*;')
        self.methodDecl     = re.compile(r'\s*public\s*(?P<retType>[a-zA-Z]\w*\s*(\[\s*\])?)\s*(?P<func>[a-zA-Z]\w*)\s*\((?P<arglist>.*)\).*;\s*$')
        self.partMethodDecl = re.compile(r'\s*public\s*(?P<retType>[a-zA-Z]\w*\s*(\[\s*\])?)\s*(?P<func>[a-zA-Z]\w*)\s*\((?P<arglist>.*)[\);]?\s*$')

        return
    
    def reset (self) :
        
        # dicts to store references to other AIDA-Interfaces, include-files
        # for STL components (e.g., string, vector) and the base-classes
        # use dicts here, as these have methods to query whether a key exists
        self.refs  = {}
        self.incs  = {}
        self.bases = {}

        # the phase is used to fill the different lists, as there is some info
        # which needs to be filled _before_ the class statement, although the
        # information is determined during analysis. Possible phases are:
        # - pre   : all before the "public interface" ("class") statement
        # - class : inside the class definition (possibly also "method")
        # - post  : things after the class definition
        self.phase = "pre"

        # arrays of lines collected during the various "analysis" phases
        self.pre = []
        self.cls = []
        self.pst = []

        # the input file is read in one go ...
        self.allLines = []
        self.index    = 0
        self.nextLine = ""

        return
    # --------------------------------------------------------------------------------    
    # Maps some java types to their corresponding C++ types:
    # "String" gets converted into "std::string",
    # "boolean" into "bool"
    # as special cases, "Class" and "Object" get transformed into "const std::string &" and "INtupleEntry", respectively
    def transformType(self,_inVal) :
        inVal = string.strip(_inVal)
        inVal = self.chkVector(inVal)
        newType = { "String"  : "const std::string &",
                    "Object"  : "INtupleEntry",
                    "Class"   : "const std::string &",
                    "void"    : "bool",
                    "boolean" : "bool"}
        retVal = inVal
        if (newType.has_key(inVal)) :
            retVal = newType[inVal]
        return retVal
    # --------------------------------------------------------------------------------    
    # check if an arg is a "xxx []" which then will be transformed into
    # std::vector< xxx > ... possibly transformed into a "const ... &" later
    # Class is mapped to "const std::string &" 
    #
    # FIXME: should we return std::vector< std::string > instead of
    #                         std::vector< const std::string & > ???
    #
    def chkVector(self, arg) :
        retVal = arg
        vectorDecl  = re.compile(r'\s*(?P<vectName>[a-zA-Z]*\s*)\[\][,\)\s]*')
        vectorMatch = vectorDecl.match(arg)
        if (vectorMatch) :
            if (not self.incs.has_key("vector") ) :
                self.incs["vector"] = 1

            typeMap = { "String"  : "std::string",
                        "Class"   : "const std::string &",
                        "boolean" : "bool"}

            newType = vectorMatch.group("vectName") 
            if (typeMap.has_key(newType)) :
                newType = typeMap[newType]
            retVal = "const std::vector< " + newType + " > &"
            # self.refs[retVal] = 1
        return retVal
    # --------------------------------------------------------------------------------    
    # Return types are converted according to their initial type.
    # As a rule, all Interface classes are converted into their pointers.
    # void is converted into "bool" to allow for checks on return type
    # As a side-effect, the dicts for refs and incs are filled
    def transRetType(self,_inVal) :
        retVal = self.transformType(_inVal)
        if (retVal[0] == "I") :
            if (not self.refs.has_key(retVal) ) : self.refs[retVal] = 1
            return retVal + " *"
        else:
            return retVal

    # --------------------------------------------------------------------------------    
    # Similar to return types, the argument types need transformation.
    # For arguments, Interface types are converted into "const Ixxx &",
    # As a side-effect, the dicts for refs and incs are filled
    def transArgType(self, _inVal) :
        retVal = self.transformType(_inVal)
        if (retVal[0] == "I") :
            if (not self.refs.has_key(retVal) ) : self.refs[retVal] = 1
            return "const " + retVal + " &"
        else:
            return retVal
    # --------------------------------------------------------------------------------    
    # transform the full argument list 
    def transArgList(self, _inVal) :
        retVal = ""
        args = string.split(_inVal, ",")
        if ( len(args) < 1  or
             _inVal == ");" or
             _inVal == "" ) :
            return ""
    
        for arg in args :
            words = string.split(arg)
            if ( len(words) != 2 ) :
                print "????? ", args, words
            else:
                retVal += self.transArgType(words[0]) + " " + words[1]
            if (arg != args[-1]) :
                retVal += ", "
                
        return retVal
    # --------------------------------------------------------------------------------    
    def readNextLine(self) :
        self.nextLine = self.allLines[self.index][:-1]
        self.index   += 1
        return
    # --------------------------------------------------------------------------------    
    # convert a line by comparing it with the regexps defined for the various
    # matching cases ... in case of multi-line statements, collect more lines
    # until the statement is valid.
    # NOTE: presently it's assumed that only class declaration and method declaration
    #       statements can be multi-line. It is further assumed that there is no trailing 
    #       code after the "closing" signature ( "{" for class-decl, ";" for method-decl )
    
    def convertNextLine(self) :
        self.readNextLine()
        inLine        = self.nextLine

        stringDecl = re.compile(r'.*String\s*.*')
        if (stringDecl.match(inLine) and not self.incs.has_key("string") ) :
            self.incs["string"] = 1
            
        matchDecl       = self.classDecl.match (inLine)
        matchPartDecl   = self.classPartDecl.match (inLine)
        matchClEnd      = self.classEnd.match  (inLine)
        matchRemove     = self.removeDecl.match(inLine)
        matchMethod     = self.methodDecl.match(inLine)
        matchPartMethod = self.partMethodDecl.match(inLine)
    
        if (matchDecl) :
            outLine = self.convertClassDecl(inLine)
            
        elif (matchPartDecl and not matchDecl) :
            # print "checking for ; in:", inLine
            matchClassStart = re.compile(r'.*{\s*$')

            while ( self.index < len(self.allLines) ) :
                self.readNextLine()
                inLine         = inLine + string.lstrip(self.nextLine)
                if ( matchClassStart.match(inLine) ) :
                    break
                
            # re-match ...
            matchDecl = self.classDecl.match(inLine)

            if ( not matchDecl ) :
                raise "decl: match failed after collecting full line :'" + inLine + "'"
            else :
                outLine = self.convertClassDecl(inLine)

        elif (matchMethod) :
            outLine = self.convertMethod(inLine)

        elif (matchPartMethod and not matchMethod) :

            # print "checking for ; in:", inLine
            matchEndMethod = re.compile(".*;\s*$")

            while ( self.index < len(self.allLines) ) :
                self.readNextLine()
                inLine         = inLine + string.lstrip(self.nextLine)
                if ( matchEndMethod.match(inLine) ) :
                    break
                
            # re-match ...
            matchMethod = self.methodDecl.match(inLine)

            if ( not matchMethod ) :
                raise "meth: match failed after collecting full line :'" + inLine + "'"
            else :
                outLine = self.convertMethod(inLine)

        elif (matchClEnd) :
            self.phase = "post"
            outLine  = "};\n"

        elif (matchRemove) :
            outLine = ''

        else :
            outLine = inLine + "\n"

        return outLine
    # --------------------------------------------------------------------------------
    # helper method to do the conversion of a class declaration (incl. treatment of
    # inheritance). Will be called with a "complete" statement
    def convertClassDecl(self, inLine) :
        self.phase = "class"
        
        # re-match ...
        matchDecl = self.classDecl.match(inLine)
        
        # print matchDecl.groupdict()
        outLine  = "\nclass I" + matchDecl.group("clName")
        if ( matchDecl.group("baseNames") != None ) :
            outLine += " : "
            bases = string.split( matchDecl.group("baseNames") )
            for cls in bases :
                outLine += " virtual public I" + cls
                if (cls != bases[-1]) : outLine += ", "
                self.bases["I"+cls] = 1
        outLine += " {"
        outLine += "\n\npublic:\n"
        
        # add virtual inline destructor ...
        outLine += "\n   virtual ~I" + matchDecl.group("clName") + "() { /* nop */; }\n"

        return outLine
    # --------------------------------------------------------------------------------    
    # helper method to do the conversion of a method declaration (incl. treatment of
    # arguments). Will be called with a "complete" statement
    def convertMethod(self, inLine) :

        oldPhase = self.phase
        self.phase = "method"
        
        # re-match ...
        matchMethod = self.methodDecl.match(inLine)

        if ( re.compile(r'.*\bbyte\s*.*').match(inLine) ) :
            return "// unknown parameter 'byte' found: " + inLine

        # print matchMethod.groups()
        outLine  = "   virtual "
        outLine += self.transRetType(matchMethod.group("retType")) + " "
        outLine += matchMethod.group("func") + " ( "
        outLine += self.transArgList(matchMethod.group("arglist"))
        outLine += " ) = 0;\n"
        
        oldPhase = self.phase
        return outLine
    
    # --------------------------------------------------------------------------------    
    # main steering ...
    def java2hdr (self, fileName) :
        # reset internal arrays and dicts
        self.reset()

        # determine base file name
        fileBase  = string.split(fileName,".")[0]
        fullLine = ""
        multiLine = ""

        # open java file, read it into memory and close it
        jFile = open(fileName,"r")
        self.allLines = jFile.readlines()
        jFile.close()

        # "analyse" ...
        while (self.index < len(self.allLines) ) :

            newLine = self.convertNextLine()
            
            if (self.phase == "pre") :
                self.pre.append(newLine)
            elif (self.phase == "class" or self.phase == "method") :
                self.cls.append(newLine)
            elif (self.phase == "post") :
                self.pst.append(newLine)

        # ------------------------------------------ open .h file and start writing the disclaimer ...
        hFileName = fileBase + ".h"
        hFile = open(hFileName, "w")
        
        hFile.write("\n// This file has been automatically converted from " + fileBase + ".java \n")
        hFile.write("\n#ifndef AIDA_" + string.upper(fileBase) + "_H \n")
        hFile.write("#define AIDA_" + string.upper(fileBase) + "_H 1\n")

        # ------------------------------------------ ... preamble, comments ...
        for newLine in self.pre :
            hFile.write(newLine)

        # ------------------------------------------ ... includes for string, vector, and the like ...
        if ( len(self.incs.keys()) > 0 ) : 
             hFile.write("\n// includes \n")
             for inc in self.incs.keys() :
                 hFile.write('#include <' + inc + '>\n')

        # ------------------------------------------ ... type ahead declarations for used types ...
        if ( len(self.refs.keys()) > 0 ) : 
            hFile.write("\n// type ahead declarations \n")
            for ref in self.refs.keys() :
                hFile.write("class " + ref + ";\n")

        # ------------------------------------------ ... includes for base-classes
        if ( len(self.bases.keys()) > 0 ) : 
            hFile.write("\n// base class(es)  \n")
            for base in self.bases.keys() :
                hFile.write('#include "AIDA/' + base + '.h"\n')

        # ------------------------------------------ ... class body ...
        hFile.write("\n")
        for newLine in self.cls :
            hFile.write(newLine)

        # ------------------------------------------ ... stuff at the end ...
        for newLine in self.pst :
            hFile.write(newLine)

        # ------------------------------------------ ... write trailer and close ...
        hFile.write("#endif /* ifndef AIDA_" + string.upper(fileBase) + "_H */ \n")
        hFile.write("\n")
        
        hFile.close()
    
        return
# --------------------------------------------------------------------------------        


if (__name__ == "__main__") :

    cnv = Java2CPPConverter()

    if (len(sys.argv) > 2) :
        for arg in sys.argv[1:] :
            print "processing: ", arg, 
            try :
                cnv.java2hdr(arg)
            except :
                print " <-- Error"
                pass
            else :
                print
    else:
        cnv.java2hdr(sys.argv[1])
        
