#ifndef AIDA_IPLOTTER_H 
#define AIDA_IPLOTTER_H 1

/**
 * User level interface to Plotter.
 * This IPlotter interface assumes the handling of 
 * multiple plotting regions. Then the
 * IPlotter should be seen as a "page" managing multiple
 * drawing region (and 
 * should probably inherit some IPage interface).
 * It assumes that a "current" region mechanism is used.
 * The keyword "region" is borrowed 
 * from the OpenInventor terminology (viewing region).
 * 
 * @author Guy Barrand
 * @version $Id: IPlotter.h,v 1.5 2001/11/29 10:23:06 barrand Exp $
 */

// includes 
#include <string>

// type ahead declarations 
class IHistogram;
class ICloud;
class IFunction;

class IPlotter {
public:
  virtual ~IPlotter() { /* nop */; }

  /**
   * createRegion, createRegions are the region factories.
   */

  /**
   * Create a new region.
   * @param x, y, w, h : position and size in normal coordinates
   * that is to say between 0 and 1. Then a region with (x=0,y=0,w=1,h=1) maps
   * the full page. "x" goes left to right. "y" bottom to top.
   * @return The index of the region. IConstants::NOT_FOUND is returned in case
   * of failure.
   */
  virtual int createRegion(double x = 0,double y = 0, 
			   double w = 1, double h = 1) = 0;

  /**
   * Create a grid of regions.
   * By default the current region is the top left one (with index 0).
   * Region are indexed from zero, with increasing order
   * left to right, top to bottom.
   * @param index set the current region.
   */
  virtual bool createRegions(int columns = 1, int rows = 1, int index = 0) = 0;

  /**
   * Add a data analysis object (histogram, function, cloud,...)
   * in the list of things to plot in the current region.
   * Activate the underlying graphic toolkit in order to bring
   * something in the screen window(s) attached to the plotter.
   * It assumes that the plotter had been mapped on the
   * screen with the show method.
   */
  virtual bool plot(const IHistogram&) = 0;
  virtual bool plot(const IFunction&) = 0;
  virtual bool plot(const ICloud&) = 0;

  /**
   *  Remove a data analysis object in the list of things to plot 
   * in the current region.
   *  Activate the graphic layer to update the screen window.
   */
  virtual bool remove(const IHistogram&) = 0;
  virtual bool remove(const IFunction&) = 0;
  virtual bool remove(const ICloud&) = 0;
    
  /**
   * Set current region to be the "next" one.
   * If exceeding the number of regions, 
   * the current region index return to 0.
   */ 
  virtual bool next() = 0;

  /**
   * Set current region by giving its index [0,n-1]
   */ 
  virtual bool setCurrentRegion(int = 0) = 0;

  /**
   * @return current region index.
   */ 
  virtual int currentRegion() const = 0;

  /**
   * Clear the list of things to plot for the current region.
   * Clear the corresponding area on screen window(s).
   */
  virtual bool clearRegion() = 0;

  /**
   *  Clear all regions. It does not reset the number 
   * and position of regions. This can be done through
   * a setRegions(1,1).
   */
  virtual bool clearPage() = 0;
    
  /*
   *  Set various plotting paramters for the current region
   * or the whole page.
   * Activate the graphic layer and update the screen window(s) 
   * if needed.
   */
  virtual bool setParameter(const std::string& parameter,
			    const std::string& value) = 0;
    
  /**
   * Produce an output file.
   * @param type  A string to describe the type of the output : 
   *  "PS"  or "PostScript" for PostScript, "JPEG" for JPEG.
   *  If nothing is given, the type is guessed
   *  from the file suffix (.ps for PostScript,
   *  .jpg for JPEG, etc...).
   */
  virtual bool write(const std::string& filename,
		     const std::string& type = "") = 0;

  /*
   * GUI related methods :
   */

  /**
   * Map the plotter on the screen.
   */
  virtual bool show() = 0;
    
  /**
   * Unmap the plotter on the screen.
   */
  virtual bool hide() = 0;
    
  /**
   * Refresh the screen window(s).
   * Used when a plotted data analysis object
   * received new data.
   */
  virtual bool refresh() = 0;
    
  /**
   * Give control to the plotter GUI. The plotter must be able 
   * to return from this function from a user action in its GUI.
   */
  virtual bool interact() = 0;
    
};

#endif /* ifndef AIDA_IPLOTTER_H */ 
