package hep.aida;

/**
 * User level interface to a Tuple.
 * 
 * @author Pavel Binko, Dino Ferrero Merlino, Wolfgang Hoschek
 * @author Tony Johnson, Andreas Pfeiffer, Mark Donszelmann, Guy Barrand
 * @version $Id: ITuple.java,v 1.5 2001/11/12 11:33:27 barrand Exp $
 */
public interface ITuple {
    
    /** The label for this n-tuple */
    public String label();
    
    // writing interface, always at the end
    public void fill(int column, double value) throws IllegalArgumentException;
    public void fill(int column, float value) throws IllegalArgumentException;
    public void fill(int column, int value) throws IllegalArgumentException;
    public void fill(int column, short value) throws IllegalArgumentException;
    public void fill(int column, long value) throws IllegalArgumentException;
    public void fill(int column, char value) throws IllegalArgumentException;
    public void fill(int column, byte value) throws IllegalArgumentException;
    public void fill(int column, boolean value) throws IllegalArgumentException;
    public void fill(int column, String value) throws IllegalArgumentException;
    public void fill(int column, Object value) throws IllegalArgumentException;
    
    public void fill(double[] values) throws IllegalArgumentException;
    public void fill(float[] values) throws IllegalArgumentException;
    /**
     * Should be called after fill is called for the columns.
     * Unfilled columns will be filled with the default value for that column.
     */
    public void addRow() throws OutOfStorageException;
    /**
     * Clears the current (not yet added) row.
     */
    public void resetRow();
    /**
     * Clears all rows from ntuple?
     */
    public void reset();
    
    // reading interface
    /**
     * The number of rows currently in the ntuple.
     * @returns -1 if cannot be determined.
     */
    public int rows();
    /**
     * Positions the read cursor immediately before the first row.
     */
    public void start();
    /**
     * Skips rows.
     * @param rows number of rows to skip, greater than 0.
     * @returns false if it cannot skip rows.
     */
    public boolean skip(int rows);
    /**
     * Positions the cursor at the next row.
     * @returns false if there is no next row.
     */
    public boolean next();
    /**
     * Convert a name to a column index.
     * Note: C++ version may return -1 if column not found
     * @returns the column number for a given name 
     */
    public int findColumn(String name) throws IllegalArgumentException;
    
    public double getDouble(int column) throws ClassCastException;
    public float getFloat(int column) throws ClassCastException;
    public int getInt(int column) throws ClassCastException;
    public short getShort(int column) throws ClassCastException;
    public long getLong(int column) throws ClassCastException;
    public char getChar(int column) throws ClassCastException;
    public byte getByte(int column) throws ClassCastException;
    public boolean getBoolean(int column) throws ClassCastException;
    public String getString(int column) throws ClassCastException;
    public Object getObject(int column) throws ClassCastException;
    
    /** Number of columns in ntuple */
    public int columns();
    /** Gets the name of a column from its index */
    public String columnName(int column);
    /** Returns the type of the column (maybe a String in C++) */
    public Class columnType(int column);
    // values can be approximations, or can return Double.NaN
    public double columnMin(int column);
    public double columnMax(int column);
    public double columnMean(int column);
    public double columnRms(int column);
    
    // The following methods are under discussion, and will likely
    // be unimplemented in AIDA 2.x
    //public void setFilter(IFilter filter);
    //public void addEvaluator(String name, Class type,  IEvaluator evaluator);


  /**
   * Fill an IHistogram1D from the tuple.
   * @param histogram. The histogram to fill.
   * @param evaluator. The evaluator object used to compute the value
   *                   to be filled in the histogram. The value is computed
   *                   from the current row of the tuple.
   * @param filter. The filter object used to accept or not the current row.
   *                If rejected, the histogram is not filled.
   */
   public boolean project(IHistogram1D histogram,
			  IEvaluator evaluatorX,
			  IFilter filter);

  /**
   * Fill an IHistogram2D from the tuple.
   * @param histogram. The histogram to fill.
   * @param evaluatorX. The evaluator object used to compute the "x value"
   *                   to be filled in the histogram.
   * @param evaluatorY. The evaluator object used to compute the "y value"
   *                   to be filled in the histogram.
   *                   Then some : fill(valueX, valueY, 1) is done for each 
   *                   tuple row.
   * @param filter. The filter object used to accept or not the current row.
   *                If rejected, the histogram is not filled.
   */
   public boolean project(IHistogram2D histogram,
			  IEvaluator evaluatorX,
			  IEvaluator evaluatorY,
			  IFilter filter);
   
  /**
   * Fill an IHistogram3D from the tuple.
   */
   public boolean project(IHistogram3D histogram,
			  IEvaluator evaluatorX,
			  IEvaluator evaluatorY,
			  IEvaluator evaluatorZ,
			  IFilter filter);
   
}
